@@ -1,7 +1,9 @@
-import zipfile
-import sys, os
 import codecs
+import os
+import sys
 import subprocess
+import zipfile
+
 from StringIO import StringIO
 
 import Image
@@ -22,9 +24,9 @@ except ImportError:
         path_list = os.path.abspath(path).split(os.path.sep)
         # Work out how much of the filepath is shared by start and path.
         i = len(os.path.commonprefix([start_list, path_list]))
-        rel_list = [os.path.pardir] * (len(start_list)-i) + path_list[i:]
+        rel_list = [os.path.pardir] * (len(start_list) - i) + path_list[i:]
         if not rel_list:
-            return curdir
+            return path
         return os.path.join(*rel_list)
 
 usage = """
@@ -41,6 +43,7 @@ parser.add_option('--debug', dest='debug', action='store_true')
 parser.add_option('--download-source-link', dest='download_source_link',
     action='store_true', default=False)
 
+
 class RstFile(object):
     headings = ['=', '-', '~', "^", '"', "'"]
 
@@ -81,7 +84,7 @@ class RstFile(object):
             line = indent + line
         self._addline(line)
         self.line_started = True
-            
+
     def append(self, line=''):
         self.current._append(line)
 
@@ -101,14 +104,14 @@ class RstFile(object):
 
     def _heading(self, text, depth):
         self.append(text)
-        self.append(self.headings[depth]*len(text))
+        self.append(self.headings[depth] * len(text))
         self.append()
 
     def start_bloc(self, indent=4):
         self.current._start_bloc(indent)
 
     def _start_bloc(self, indent=4):
-        self.indents.append(' '*indent)
+        self.indents.append(' ' * indent)
 
     def end_bloc(self):
         self.current._end_bloc()
@@ -158,6 +161,7 @@ class RstFile(object):
 
     def start_table_header(self):
         pass
+
     def end_table_header(self):
         self.current_table.append('separator')
 
@@ -172,11 +176,10 @@ class RstFile(object):
                     last_line_i -= 1
                 last_line = self.current_table[last_line_i]
                 if last_line[i][2] > 1:
-                    row.append((None, last_line[i][1], last_line[i][2]-1))
+                    row.append((None, last_line[i][1], last_line[i][2] - 1))
                 else:
                     break
 
-
     def start_row(self):
         self.current_table.append([])
         self.insert_rows_spanned_cells()
@@ -191,10 +194,10 @@ class RstFile(object):
         row.append((self.current, columns_span, rows_span))
 
         for i in xrange(1, columns_span):
-            row.append((None, columns_span-i, rows_span))
+            row.append((None, columns_span - i, rows_span))
 
         self.insert_rows_spanned_cells()
-           
+
     def end_cell(self):
         self.current = self
 
@@ -202,8 +205,8 @@ class RstFile(object):
         self.current_table = []
 
     def end_table(self):
-        column_widths = [0]*len(self.current_table[0])
-        row_heights = [0]*len(self.current_table)
+        column_widths = [0] * len(self.current_table[0])
+        row_heights = [0] * len(self.current_table)
 
         rendered_cells = []
         separator_index = -1
@@ -217,18 +220,21 @@ class RstFile(object):
                 if cell[0] is None:
                     rendered_row.append(cell)
                 else:
-                    rendered_cell = cell and cell[0].getvalue().split('\n') or []
+                    rendered_cell = cell \
+                                    and cell[0].getvalue().split('\n') \
+                                    or []
                     rendered_row.append((rendered_cell, cell[1], cell[2]))
 
                     column_widths[cell_i] = max(column_widths[cell_i],
-                                                *[len(line) for line in rendered_cell])
+                                                * [len(line)
+                                                   for line in rendered_cell])
                     row_heights[row_i] = max(row_heights[row_i],
                                              len(rendered_cell))
 
             rendered_cells.append(rendered_row)
 
         if separator_index != -1:
-            del row_heights[separator_index+1]
+            del row_heights[separator_index + 1]
 
         def h_border(char='-', skip_columns_groups=[], skip_nodes=[]):
             skip_columns = []
@@ -242,16 +248,17 @@ class RstFile(object):
                 line = '+'
             for col_i, colw in enumerate(column_widths):
                 if col_i in skip_columns:
-                    line += ' ' * (colw+2)
+                    line += ' ' * (colw + 2)
                     if col_i in group_last_columns:
-                        if col_i + 1 in skip_columns or col_i + 1 == len(column_widths):
+                        if col_i + 1 in skip_columns \
+                                or col_i + 1 == len(column_widths):
                             line += '|'
                         else:
                             line += '+'
                     else:
                         line += ' '
                 else:
-                    line += char * (colw+2)
+                    line += char * (colw + 2)
                     if col_i in skip_nodes:
                         line += char
                     else:
@@ -271,7 +278,9 @@ class RstFile(object):
                 line = '| '
                 for cell_i, cell in enumerate(row):
                     if cell[0] is not None and len(cell[0]) > line_i:
-                        line += cell[0][line_i] + ' '*(column_widths[cell_i]-len(cell[0][line_i]))
+                        line += cell[0][line_i] \
+                            + ' ' * (column_widths[cell_i]
+                                     - len(cell[0][line_i]))
                     else:
                         line += ' ' * column_widths[cell_i]
                     if cell[1] == 1:
@@ -294,14 +303,16 @@ class RstFile(object):
             for i, cell in enumerate(row):
                 next_row_i = row_i + 1
                 if cell[1] > 1 and (
-                        row_i == len(rendered_cells) - 1 
+                        row_i == len(rendered_cells) - 1
                         or rendered_cells[next_row_i][i][1] > 1):
                     skip_nodes.append(i)
 
             next_border_char = '-'
             if row_i == separator_index:
                 next_border_char = '='
-            border = h_border(char=next_border_char, skip_columns_groups=skip_columns_groups, skip_nodes=skip_nodes)
+            border = h_border(char=next_border_char,
+                              skip_columns_groups=skip_columns_groups,
+                              skip_nodes=skip_nodes)
             self.append(border)
 
         self.append()
@@ -317,12 +328,14 @@ class RstFile(object):
                 kw['width'] = width
             if height:
                 kw['height'] = height
-            self.simpledirective('|%s| image' % fname.replace('.', '_'), 'images/' + fname, **kw)
+            self.simpledirective('|%s| image' % fname.replace('.', '_'),
+                                 'images/' + fname, **kw)
 
         output = self.lines
         self.lines = lines
         return u'\n'.join(output)
 
+
 class SphinxWriter(object):
     def __init__(self, title, dir, options=None):
         self.dir = dir
@@ -348,7 +361,8 @@ class SphinxWriter(object):
         self.current_file.append(text)
 
     def add_image(self, fname, fcontent, width=None, height=None):
-        self.current_file.add_image(fname, fcontent, width=width, height=height)
+        self.current_file.add_image(fname, fcontent,
+                                    width=width, height=height)
 
     def add_text(self, text, font_style=None):
         if font_style == 'italic':
@@ -366,7 +380,8 @@ class SphinxWriter(object):
         if self.current_file == self.files['index']:
             self.current_file.directive('toctree', maxdepth=2)
         self.current_file = RstFile()
-        fname = text.replace(' ', '').replace('/', '').replace(':', '_').encode('ascii', 'ignore')
+        fname = text.replace(' ', '').replace('/', '').replace(':', '_')
+        fname = fname.encode('ascii', 'ignore')
         self.files[fname] = self.current_file
 
         self.files['index'].append(fname)
@@ -411,7 +426,7 @@ class SphinxWriter(object):
         self.current_file.end_bloc()
 
     def finalise(self):
-        if len(self.files)>1:
+        if len(self.files) > 1:
             self.files['index'].end_bloc()
 
     def writeout(self):
@@ -455,6 +470,7 @@ class SphinxWriter(object):
                     f.write(fcontent)
                     f.close()
 
+
 class ODTReader(object):
     style_map = {
         'Heading': 'h0',
@@ -495,10 +511,11 @@ class ODTReader(object):
         formx="urn:openoffice:names:experimental:ooxml-odf-interop:xmlns:form:1.0",
         )
 
-    def __init__(self, file, visitor, debug=False):
+    def __init__(self, filename, visitor, debug=False):
         self.debug = debug
-        if isinstance(file, basestring):
-            file = open(file, 'rb')
+        self.filename = os.path.abspath(filename)
+        self.dirname = os.path.dirname(self.filename)
+        file = open(filename, 'rb')
         self.archive = zipfile.ZipFile(file)
         self.xml_content = self._load_xml('content.xml')
         self.xml_styles = self._load_xml('styles.xml')
@@ -531,7 +548,7 @@ class ODTReader(object):
             style = dict(name=name)
             for aname in ('display-name', 'family', 'parent-style-name',
                           'next-style-name', 'class'):
-                style[aname] = node.get('{%(style)s}'%self.ns + aname)
+                style[aname] = node.get('{%(style)s}' % self.ns + aname)
 
             for child in node.getchildren():
                 if child.tag == '{%(style)s}text-properties' % self.ns:
@@ -539,8 +556,8 @@ class ODTReader(object):
                     if font_style in ('italic', 'bold'):
                         style['font-style'] = font_style
 
-            # Special case. We know that the "Source_xx_Text" styles are for source
-            # code.
+            # Special case. We know that the "Source_xx_Text" styles
+            # are for source code.
             if name.startswith("Source_") and name.endswith("_Text"):
                 style['font-style'] = 'fixed'
 
@@ -568,7 +585,7 @@ class ODTReader(object):
                     h = None
             if h is not None:
                 self.style_map[key] = 'h%s' % h
-            if display_name.lower() in ['note']:
+            if display_name.lower() in ['note', 'information']:
                 self.style_map[key] = 'note'
             if display_name.lower() in ['warning', 'avertissement']:
                 self.style_map[key] = 'warning'
@@ -591,10 +608,10 @@ class ODTReader(object):
 
     def recurse_node(self, node):
         font_style = None
-        style_name = node.get('{%(text)s}style-name'%self.ns)
+        style_name = node.get('{%(text)s}style-name' % self.ns)
         style = self.styles.get(style_name)
         if style:
-            font_style = style.get('font-style') 
+            font_style = style.get('font-style')
         if node.text:
             self.call_visitor('on_text', node.text, font_style=font_style)
         for child in node.getchildren():
@@ -605,9 +622,12 @@ class ODTReader(object):
     def get_node_kind(self, node):
         if node is None:
             return
-        style_name = node.get('{%(text)s}style-name'%self.ns)
+        if node.tag == '{%(text)s}list-item' % self.ns and len(node) > 0:
+            node = node[0]
+        style_name = node.get('{%(text)s}style-name' % self.ns)
         style = self.styles[style_name]
-        if style and style_name not in self.style_map and style['parent-style-name'] in self.style_map:
+        if style and style_name not in self.style_map \
+                and style['parent-style-name'] in self.style_map:
             style_name = style['parent-style-name']
             style = self.styles[style_name]
         if style_name in self.style_map:
@@ -625,7 +645,11 @@ class ODTReader(object):
         fname = href.split('/')[-1]
         if href.startswith('./'):
             href = href[2:]
-        fcontent = self.archive.read(href)
+        if href.startswith('../'):
+            fcontent = open(
+                os.path.join(self.dirname, href[3:]), 'r').read()
+        else:
+            fcontent = self.archive.read(href)
         return fname, fcontent
 
     def handle_node(self, node):
@@ -635,13 +659,15 @@ class ODTReader(object):
             self.recurse_node(node)
         if node.tag == '{%(draw)s}frame' % self.ns:
             children = node.getchildren()
-            if len(children) == 1 and children[0].tag == '{%(draw)s}image' % self.ns:
+            if len(children) == 1 \
+                    and children[0].tag == '{%(draw)s}image' % self.ns:
                 fname, fcontent = self.get_image_properties(children[0])
                 if fname is None:
                     return
                 width = node.get('{%(svg)s}width' % self.ns)
                 height = node.get('{%(svg)s}height' % self.ns)
-                self.call_visitor('on_image', fname, fcontent, width=width, height=height)
+                self.call_visitor('on_image', fname, fcontent,
+                                  width=width, height=height)
             else:
                 self.recurse_node(node)
 
@@ -660,9 +686,12 @@ class ODTReader(object):
                 self.recurse_node(node)
                 self.end_visit('list')
         if node.tag == '{%(text)s}list-item' % self.ns:
+            kind = self.get_node_kind(node)
+            self.start_visit(kind)
             self.start_visit('list_item')
             self.recurse_node(node)
             self.end_visit('list_item')
+            self.end_visit(kind)
         if node.tag in (
                 '{%(text)s}p' % self.ns,
                 '{%(text)s}h' % self.ns):
@@ -679,7 +708,8 @@ class ODTReader(object):
 
         if node.tag == '{%(table)s}table-column' % self.ns:
             self.call_visitor('table_column',
-                repeat=int(node.get('{%(table)s}number-columns-repeated'%self.ns, 1)))
+                repeat=int(node.get(
+                    '{%(table)s}number-columns-repeated' % self.ns, 1)))
 
         if node.tag == '{%(table)s}table-header-rows' % self.ns:
             self.start_visit('table_header_rows')
@@ -693,8 +723,10 @@ class ODTReader(object):
 
         if node.tag == '{%(table)s}table-cell' % self.ns:
             self.start_visit('table_cell',
-                columns_span=int(node.get('{%(table)s}number-columns-spanned'%self.ns, 1)),
-                rows_span=int(node.get('{%(table)s}number-rows-spanned'%self.ns, 1)),
+                columns_span=int(node.get(
+                    '{%(table)s}number-columns-spanned' % self.ns, 1)),
+                rows_span=int(node.get(
+                    '{%(table)s}number-rows-spanned' % self.ns, 1)),
             )
             self.recurse_node(node)
             self.end_visit('table_cell')
@@ -703,7 +735,8 @@ class ODTReader(object):
             self.call_visitor('covered_table_cell')
 
     def iterate(self):
-        nodes = self.xml_content.findall('{%(office)s}body/{%(office)s}text/*' % self.ns)
+        nodes = self.xml_content.findall(
+            '{%(office)s}body/{%(office)s}text/*' % self.ns)
 
         for i, el in enumerate(nodes):
             self.handle_node(el)
@@ -718,6 +751,7 @@ class ODTReader(object):
         except StopIteration:
             self.done = True
 
+
 class Converter(object):
     def __init__(self, source, targetdir, debug=False, options=None):
         self.options = options or {}
@@ -835,7 +869,6 @@ class Converter(object):
         self.writer.h6(self._get_buffer())
 
     def on_list_start(self):
-        self.set_last_bloc()
         pass
 
     def on_list_end(self):
@@ -870,19 +903,22 @@ class Converter(object):
 
     def on_table_header_rows_start(self):
         self.writer.start_table_header()
+
     def on_table_header_rows_end(self):
         self.writer.end_table_header()
 
     def on_table_row_start(self):
         self.writer.start_row()
+
     def on_table_row_end(self):
         self.writer.end_row()
-        
+
     def on_table_cell_start(self, columns_span=1, rows_span=1):
         self.writer.start_cell(columns_span=columns_span, rows_span=rows_span)
+
     def on_table_cell_end(self):
         self.writer.end_cell()
-        
+
     def dostep(self):
         self.reader.dostep()
         if self.reader.done:
@@ -904,9 +940,10 @@ def convert_odt(filename, targetdir, options=None, debug=False):
     while not converter.done:
         converter.dostep()
 
+
 def main():
     options, args = parser.parse_args()
-    
+
     if len(args) == 1 and args[0].endswith('.odt'):
         args.append(args[0][:-4])
     if len(args) == 2:
@@ -935,4 +972,3 @@ def main():
 
 if __name__ == '__main__':
     main()
-
